;;; tinydebian.el --- Debian utilities.
;; $Id: tinydebian.el,v 1.55 2006/02/24 18:14:49 jaalto Exp $

;;{{{ Id

;; Copyright (C)    2001-2006 Jari Aalto
;; Keywords:        extensions
;; Author:          Jari Aalto
;; Maintainer:      Jari Aalto
;;
;; To get information on this program, call M-x tinydebian-version.
;; Look at the code with folding.el

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ........................................................ &t-install ...
;;   Put this file on your Emacs-Lisp load path, add following into your
;;   $HOME/.emacs startup file
;;
;;      (add-hook 'tinydebian-:load-hook 'tinydebian-install)
;;      (add-hook 'find-file-hooks 'tinydebian-bts-mode-maybe-turn-on)
;;      (require 'tinydebian)
;;
;;   If you have any questions or bug report to send, use this function:
;;
;;      M-x tinydebian-submit-bug-report    send question, feedback,bug report
;;
;;  To read the documentation after file has been loaded, call
;;
;;      M-x tinydebian-version

;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...

;;; Commentary:

;;  Overview of features
;;
;;      This package contains utilities for the Debian System Administarator,
;;      to help administring Debian in daily tasks and submitting bug
;;      reports from Emacs. Learn more about debian at
;;      http://www.debian.org/
;;
;;      o   colorize /var/log files like messages, syslog etc.
;;      o   Report Debian bug with M-x ... #todo
;;
;;  Quick start

;;}}}

;;; Change Log:

;;; Code:

;;{{{ setup: libraries

(require 'tinylibm)

(eval-when-compile (ti::package-use-dynamic-compilation))

(eval-and-compile
  ;;  Forward declarations to quiet byte compiler.
  (defvar gnus-newsgroup-name)
  (defvar font-lock-mode)
  (defvar font-lock-keyword-face)
  (defvar global-font-lock-mode)
  (defvar font-lock-keywords)
  (defvar font-lock-defaults))

(ti::package-defgroup-tiny TinyDebian tinydebian-: extensions
  "Debian System administrator's grabbag of utilities.")

;;}}}
;;{{{ setup: hooks

;;; ......................................................... &v-hooks ...

(defcustom tinydebian-:load-hook nil
  "*Hook run when file has been loaded."
  :type  'hook
  :group 'TinyDebian)

(defcustom tinydebian-:find-bug-nbr-hook '(tinydebian-bug-nbr-any)
  "*Function to return Debian bug tracking number as string.
Default value is '(tinydebian-bug-nbr-any)."
  :type  'function
  :group 'TinyDebian)

(defcustom tinydebian-:load-hook nil
  "*Hook run when file has been loaded."
  :type  'hook
  :group 'TinyDebian)

(defcustom tinydebian-:browse-url-function
  (function tinydebian-browse-url-browse-url)
  "*Function to run for HTTP URLs. Default is `browse-url'.
To use text mode inside Emacs, set value to
`tinydebian-browse-url-lynx-dump' if lynx(1) is available."
  :type  'function
  :group 'TinyDebian)

;;}}}
;;{{{ setup: user config

;;; ................................................... &v-user-config ...

(defcustom tinydebian-:install-buffer-file-name-regexp
  "/debian/\\(changelog\\|.*README\\)"
  "*Activate `tinydebian-bts-mode' on buffers whose file name match regexp.
This variable is used when function `tinydebian-install' is called."
  :type  'regexp
  :group 'TinyDebian)

(defcustom tinydebian-:buffer-tiger "*Tinydebian tiger*"
  "*Buffer name where to generate tiger(1) mail report chmod fixes.
See function `tinydebian-command-audit-report-tiger'."
  :type  'string
  :group 'TinyDebian)

(defcustom tinydebian-:buffer-wnpp-alert "*Tinydebian wnpp-alert*"
  "*Buffer name where to generate wnpp-alert(1) report.
See function `tinydebian-command-wnpp-alert'."
  :type  'string
  :group 'TinyDebian)

(defcustom tinydebian-:buffer-www "*Tinydebian WWW*"
  "*Buffer name where to put WWW call results.
See `tinydebian-:browse-url-function'."
  :type  'string
  :group 'TinyDebian)

(defcustom tinydebian-:install-gnus-newsgroup-name-regexp
  "debian"
  "*Newsgroup name regexp to match to activate `tinydebian-bts-mode'."
  :type  'string
  :group 'TinyDebian)

(defface tinydebian-:warn-face
  '((((class color) (background light))
     (:background "green"))
    (((class color) (background dark))
     (:background "sea green"))
    (((class grayscale monochrome)
      (background light))
     (:background "black"))
    (((class grayscale monochrome)
      (background dark))
     (:background "white")))
  "Face used for warnings."
  :group 'TinyDebian)

;;; Color loading section  This is messy *Blech!*
;;
(defface tinydebian-:item-face
  '((((class color) (background light))
     (:foreground "green4"))
    (((class color) (background dark))
     (:foreground "green3")))
  "Face used for noticing important items."
  :group 'TinyDebian)

;;}}}
;;{{{ setup: -- private

;;; ....................................................... &v-private ...

(defvar tinydebian-:font-lock-keywords-adaptive-date nil
  "Flag to signal that current time is used to display today's log.
For exmple in /etc/syslog today's log log entries are highlighted
differently that previous days. However this must be changed in
next day, because the daye changes.

This flags says, that adaptive-date regexs must be used.")

(make-variable-buffer-local 'tinydebian-:font-lock-keywords-adaptive-date)

(defvar tinydebian-:bin-dpkg (executable-find "dpkg")
  "*Location of `dpkg' binary.")

(defvar tinydebian-:severity-list
  '(("critical"
     "Makes unrelated software on the system (or the whole system) break,
or causes serious data loss, or introduces a security hole on systems where
you install the package.")
    ("grave"
     "Makes the package in question unuseable or mostly so, or causes data
loss, or introduces a security hole allowing access to the accounts of users
who use the package.")
    ("serious"
     "Severe violation of Debian policy (that is, it violates a
\"must\" or \"required\" directive), or, in the package maintainer's
opinion, makes the package unsuitable for release.")
    ("important"
     "A bug which has a major effect on the usability of a package,
without rendering it completely unusable to everyone.")
    ("normal"
     "The default value, applicable to most bugs.")
    ("minor"
     "A problem which doesn't affect the package's usefulness, and is
presumably trivial to fix.")
    ("wishlist"
     "For any feature request, and also for any bugs that are very
difficult to fix due to major design considerations.")
    ("fixed"
     "For bugs that are fixed but should not yet be closed. This is an
exception for bugs fixed by non-maintainer uploads. Note: the "fixed"
tag should be used instead."))
  "The bug system records a severity level with each bug report.
This is set to normal by default, but can be overridden either by supplying a Severity line in the pseudo-header when the bug is submitted Severity or error.
http://www.debian.org/Bugs/Developer#severities")

(defvar tinydebian-:severity-selected nil
  "Function `tinydebian-severity-select-*' sets this to user selection.")

(defconst tinydebian-:menu-severity
  '("\
Severity: ?h)elp c)rit g)rave s)erious i)import RET-n)orm m)inor w)ish f)ixed"
    ;; NOTE: These function are automatically created, you don't find
    ;; then with C-s. See `tinydebian-install-severity-functions'
    ((?c .      ( (call-interactively 'tinydebian-severity-select-critical)))
     (?g .      ( (call-interactively 'tinydebian-severity-select-grave)))
     (?s .      ( (call-interactively 'tinydebian-severity-select-serious)))
     (?i .      ( (call-interactively 'tinydebian-severity-select-important)))
     (?n .      ( (call-interactively 'tinydebian-severity-select-normal)))
     (?\C-m .   ( (call-interactively 'tinydebian-severity-select-normal)))
     (?m .      ( (call-interactively 'tinydebian-severity-select-minor)))
     (?w .      ( (call-interactively 'tinydebian-severity-select-wishlist)))
     (?f .      ( (call-interactively 'tinydebian-severity-select-fixed)))))
  "Severity menu.

The bug system records a severity level with each bug report. This is set
to normal by default, but can be overridden either by supplying a Severity
line in the pseudo-header when the bug is submitted (see the instructions
for reporting bugs), or by using the severity command with the control
request server.

critical
    makes unrelated software on the system (or the whole system) break, or
    causes serious data loss, or introduces a security hole on systems where
    you install the package.

grave
    makes the package in question unuseable or mostly so, or causes data loss,
    or introduces a security hole allowing access to the accounts of users who
    use the package.

serious
    is a severe violation of Debian policy (that is, it violates a \"must\" or
    \"required\" directive), or, in the package maintainer's opinion, makes the
    package unsuitable for release.

important
    a bug which has a major effect on the usability of a package, without
    rendering it completely unusable to everyone.

normal
    the default value, applicable to most bugs.

minor
    a problem which doesn't affect the package's usefulness, and is presumably
    trivial to fix.

wishlist
    for any feature request, and also for any bugs that are very difficult to
    fix due to major design considerations.

fixed
    for bugs that are fixed but should not yet be closed. This is an exception
    for bugs fixed by non-maintainer uploads. Note: the \"fixed\" tag should be
    used instead.  Certain severities are considered release-critical, meaning
    the bug will have an impact on releasing the package with the stable
    release of Debian. Currently, these are critical, grave and serious.")

(defvar tinydebian-:tags-list
  '(("patch"
     "A patch or some other easy procedure for fixing the bug is included
in the bug logs. If there's a patch, but it doesn't resolve the bug
adequately or causes some other problems, this tag should not be used.")
    ("wontfix"
     "This bug won't be fixed. Possibly because this is a choice between
two arbitrary ways of doing things and the maintainer and submitter prefer
different ways of doing things, possibly because changing the behaviour
will cause other, worse, problems for others, or possibly for other reasons.")
    ("moreinfo"
     "This bug can't be addressed until more information is provided by
the submitter. The bug will be closed if the submitter doesn't provide
more information in a reasonable (few months) timeframe. This is for
bugs like "It doesn't work". What doesn't work?.")
    ("unreproducible"
     "This bug can't be reproduced on the maintainer's system.
Assistance from third parties is needed in diagnosing the cause of the problem.")
    ("help"
     "The maintainer is requesting help with dealing with this bug.")
    ("pending"
     "The problem described in the bug is being actively worked on,
i.e. a solution is pending.")
    ("fixed"
     "This bug is fixed or worked around (by a non-maintainer upload,
for example), but there's still an issue that needs to be resolved.
This tag replaces the old \"fixed\" severity.")
    ("security"
     "This bug describes a security problem in a package (e.g., bad
permissions allowing access to data that shouldn't be accessible;
buffer overruns allowing people to control a system in ways they
shouldn't be able to; denial of service attacks that should be fixed, etc).
Most security bugs should also be set at critical or grave severity.")
    ("upstream"
     "This bug applies to the upstream part of the package.")
    ("confirmed"
     "The maintainer has looked at, understands, and basically agrees
with the bug, but has yet to fix it. (Use of this tag is optional; it is
intended mostly for maintainers who need to manage large numbers of open bugs.")
    ("fixed-upstream"
     "The bug has been fixed by the upstream maintainer, but not yet
in the package (for whatever reason: perhaps it is too complicated to
backport the change or too minor to be worth bothering).")
     ("fixed-in-experimental"
      "The bug has been fixed in the package of the experimental
distribution, but not yet in the unstable distribution.")
    ("potato"
     "This bug particularly applies to the potato release of Debian.")
    ("ipv6"
     "This bug affects support for Internet Protocol version 6.")
     ("lfs"
      "This bug affects support for large files (over 2 gigabytes).")
     ("l10n"
      "This bug is relevant to the localisation of the package.")
    ("woody"
     "This bug particularly applies to the (unreleased) woody distribution.")
    ("sarge"
     "This bug particularly applies to the sarge distribution.")
    ("etch"
     "This bug particularly applies to the etch distribution.")
    ("sid"
     "This bug particularly applies to an architecture that is
currently unreleased (that is, in the sid distribution).")
    ("experimental"
     "This bug particularly applies to the experimental distribution."))
  "Each bug can have zero or more of a set of given tags.
These tags are displayed in the list of bugs when you look at a
package's page, and when you look at the full bug log.
See <http://www.debian.org/Bugs/Developer#tags>.")

(defvar tinydebian-:wnpp-buffer "*TinyDebian WNPP*"
  "WNPP question buffer.")

(defvar tinydebian-:menu-wnpp-selected nil
  "Placeholder of selection from `tinydebian-:menu-wnpp'.")

(defconst tinydebian-:menu-wnpp
  (list
   '(format
    "TinyDebian:WNPP%s 1i)tp 2o)rphan 3a)dopt 4n)ew package ?)help q)uit"
    (if tinydebian-:menu-wnpp-selected
        (format ";%s " (symbol-name tinydebian-:menu-wnpp-selected))
      ""))
   (list
    '(?1 . ( (setq tinydebian-:menu-wnpp-selected 'package)))
    '(?p . ( (setq tinydebian-:menu-wnpp-selected 'package)))
    '(?P . ( (setq tinydebian-:menu-wnpp-selected 'package)))
    '(?2 . ( (setq tinydebian-:menu-wnpp-selected 'oprhan)))
    '(?o . ( (setq tinydebian-:menu-wnpp-selected 'oprhan)))
    '(?O . ( (setq tinydebian-:menu-wnpp-selected 'oprhan)))
    '(?3 . ( (setq tinydebian-:menu-wnpp-selected 'adopt)))
    '(?a . ( (setq tinydebian-:menu-wnpp-selected 'adopt)))
    '(?A . ( (setq tinydebian-:menu-wnpp-selected 'adopt)))
    '(?4 . ( (setq tinydebian-:menu-wnpp-selected 'new)))
    '(?n . ( (setq tinydebian-:menu-wnpp-selected 'new)))
    '(?N . ( (setq tinydebian-:menu-wnpp-selected 'new)))
    ))
  ;;  This message is straight from reportbug(1)
  ;;  'apt-get install reportbug'
  "What request type? If none of these things mean anything to you, or
you are trying to report a bug in an existing package)

1 p    ITP, `Intent To Package'. Please submit a package description
       along with copyright and URL in such a report.

2 o    The package has been `Orphaned'. It needs a new maintainer as soon as
       possible.

3 a    RFA, this is a `Request for Adoption'. Due to lack of time, resources,
       interest or something similar, the current maintainer is asking for
       someone else to maintain this package. He/she will maintain it in the
       meantime, but perhaps not in the best possible way. In short: the
       package needs a new maintainer.

4 n    RFP, this is a `Request For Package'. You have found an interesting piece of
       software and would like someone else to maintain it for Debian. Please
       submit a package description along with copyright and URL in such a
       report.

q      Quit menu.
")

(defconst tinydebian-:-wnpp-template-new-package "\

* Package name    : <package>
  Version         : x.y.z
  Upstream Author : Name <somebody@example.org>
* URL             : <homepage: http://www.example.org/>
* License         : <license: GPL, LGPL, BSD, MIT/X, etc.>
  Description     : <short desc>

(Include the long description here.)
"
  "Wnpp RFP/ITP template.")

(defconst tinydebian-:wnpp-template-licences-alist
  '("GPL"
    "LGPL"
    "BSD"
    "MIT/X")
  "List of licences.")

(defvar tinydebian-:bts-email-address "bugs.debian.org"
  "Email address or Debian Bug Tracking System.")

(defvar tinydebian-:url-http-package-search
  "http://pdo.debian.net/cgi-bin/search_packages.pl?"
  "The packages Debian control URL without parameter, up to '?' token.")

(defvar tinydebian-:url-http-package-bugs
  "http://bugs.debian.org/"
  "The bugs Debian control URL without parameter, up to '/' token.")

(defvar tinydebian-:url-http-wnpp-page-main
  "http://www.debian.org/devel/wnpp"
  "The WNPP main page URL address. No trailing slash.")

(defvar tinydebian-:url-http-wnpp-page-alist
  '(("RFA" . "rfa_bypackage")
    ("O"   . "orphaned")
    ("RFH" . "help_request")
    ("RFP" . "requested")
    ("ITP" . "being_packaged"))
  "List of mapping to pages under `tinydebian-:url-http-wnpp-page-main'.")

(defvar tinydebian-:url-debian-page-alist
  "List of Debian site pages."
  '((bts-control     . "http://www.debian.org/Bugs/server-control")
    (policy          . "")
    (best-practises  . "")))

;;}}}
;;{{{ setup: -- version

;;; ....................................................... &v-version ...

;;;###autoload (autoload 'tinydebian-version "tinydebian" "Display commentary." t)
(eval-and-compile
(ti::macrof-version-bug-report
 "tinydebian.el"
 "tinydebian"
 tinydebian-:version-id
 "$Id: tinydebian.el,v 1.55 2006/02/24 18:14:49 jaalto Exp $"
 '(tinydebian-version-id
   tinydebian-:load-hook
   tinydebian-:font-lock-keywords-adaptive-date
   tinydebian-:bin-dpkg
   tinydebian-:severity-list
   tinydebian-:severity-selected
   tinydebian-:tags-list)))

;;}}}
;;{{{ Install: bindings

;;; ........................................................ &bindings ...

;; #todo:
(defun tinydebian-default-bindings ()
  "Define default key bindings to `tinydebian-mode-map'.")

(eval-and-compile

;;;###autoload (autoload 'tinydebian-bts-mode          "tinydebian" "" t)
;;;###autoload (autoload 'turn-on-tinydebian-bts-mode  "tinydebian" "" t)
;;;###autoload (autoload 'turn-off-tinydebian-bts-mode "tinydebian" "" t)
;;;###autoload (defvar tinydebian-:bts-mode-prefix-key "\C-c-")
(ti::macrof-minor-mode-wizard
 "tinydebian-bts-" " Tdeb" "\C-c-" "Tdeb" 'TinyDebian "tinydebian-:bts-"   ;1-6

  "Debian Bug Tracking System (BTS) Minor mode. With this mode you can
jump to a bug report at or near current point (using browser), send
control messages, like turning RFS into ITP, send new RFS, send new
ITP etc.

Prefix key is:

  tinydebian-:bts-mode-prefix-key

Mode description:

\\{tinydebian-:bts-mode-prefix-map}"

  "TinyDebian BTS"
 nil
 "TinyDebian BTS minor mode menu."
 (list
  tinydebian-:bts-mode-easymenu-name
  ["Report bug"                    tinydebian-bug-report-mail   t]
  ["Goto URL by bug number"        tinydebian-bug-browse-url-by-bug t]
  ["Goto URL by package bugs"      tinydebian-bug-browse-url-by-package-bugs t]
  ["Goto URL by package name"      tinydebian-bug-browse-url-by-package-name t]
  ["Installed pkgs and WNPP"       tinydebian-command-show-wnpp-alert t]

  "----"

  (list
   "WNPP messages"
   ["Send BTS ITA: intent to adopt"      tinydebian-bts-mail-type-ita t]
   ["Send BTS ITP: intent to pkg"        tinydebian-bts-mail-type-itp t]
   ["Send BTS RFA: request for adopt"    tinydebian-bts-mail-type-rfa t]
   ["Send BTS RFH: request for help"     tinydebian-bts-mail-type-rfh t]
   ["Send BTS RFP: request for packege"  tinydebian-bts-mail-type-rfp t]
   ["Send BTS RFS: request for sponsor"  tinydebian-bts-mail-type-rfs t]
   ["Send BTS O: orphan"                 tinydebian-bts-mail-type-orphan t]
   ["WNPP control menu"                  tinydebian-package-wnpp-main t])

  (list
   "WNPP www information pages"
   ["List of WNPP RFP"                   tinydebian-url-wnpp-rfp      t]
   ["List of WNPP RFH"                   tinydebian-url-wnpp-rfh      t]
   ["List of WNPP RFA"                   tinydebian-url-wnpp-rfa      t]
   ["List of WNPP Orphaned"              tinydebian-url-wnpp-orphaned t]
   ["List of WNPP ITP"                   tinydebian-url-wnpp-itp      t])

  (list
   "BTS Control messages"
   ["Send BTS Ctrl close"                tinydebian-bts-mail-ctrl-close    t]
   ["Send BTS Ctrl severity"             tinydebian-bts-mail-ctrl-severity t]
   ["Send BTS Ctrl tags"                 tinydebian-bts-mail-ctrl-tags     t]
   ["Send BTS Ctrl forward"              tinydebian-bts-mail-ctrl-forward  t]
   ["Send BTS Ctrl reassign"             tinydebian-bts-mail-ctrl-reassign t]
   ["Send BTS Ctrl retitle"              tinydebian-bts-mail-ctrl-retitle  t])

  (list
   "Debin www manuals"
   ["URL BTS Ctrl page"                  tinydebian-url-bts-ctrl-page         t]
   ["URL Policy Manual"                  tinydebian-url-policy-manual         t]
   ["URL Best practises"                 tinydebian-url-policy-best-practises t])

  )
 (progn

   (define-key map  "b"  'tinydebian-bug-browse-url-by-bug)
   (define-key map  "B"  'tinydebian-bug-browse-url-by-package-bugs)
   (define-key map  "m"  'tinydebian-bug-report-mail)
   (define-key map  "p"  'tinydebian-bug-browse-url-by-package-name)
   (define-key map  "w"  'tinydebian-package-wnpp-main)

   (define-key map  "-a" 'tinydebian-bts-mail-type-ita)
   (define-key map  "-A" 'tinydebian-bts-mail-type-rfa)
   (define-key map  "-h" 'tinydebian-bts-mail-type-rfh)
   (define-key map  "-P" 'tinydebian-bts-mail-type-itp)
   (define-key map  "-p" 'tinydebian-bts-mail-type-rfp)
   (define-key map  "-s" 'tinydebian-bts-mail-type-rfs)
   (define-key map  "-o" 'tinydebian-bts-mail-type-orphan)

   ;;  (i)nfo (i)nstalled
   (define-key map  "ii" 'tinydebian-command-show-wnpp-alert)

   ;;  List Url commands
   (define-key map  "lP"  'tinydebian-url-wnpp-itp)
   (define-key map  "lp"  'tinydebian-url-wnpp-rfp)
   (define-key map  "lh"  'tinydebian-url-wnpp-rfh)
   (define-key map  "la"  'tinydebian-url-wnpp-rfa)
   (define-key map  "lo"  'tinydebian-url-wnpp-orphaned)

   ;;  (C)ontrol commands
   (define-key map  "cc"  'tinydebian-bts-mail-ctrl-close)
   (define-key map  "cs"  'tinydebian-bts-mail-ctrl-severity)
   (define-key map  "ct"  'tinydebian-bts-mail-ctrl-tags)
   (define-key map  "cf"  'tinydebian-bts-mail-ctrl-forward)
   (define-key map  "cr"  'tinydebian-bts-mail-ctrl-reassign)
   (define-key map  "cR"  'tinydebian-bts-mail-ctrl-retitle)

   ;;  URLs
   (define-key map  "ub"  'tinydebian-url-bts-ctrl-page)
   (define-key map  "up"  'tinydebian-url-policy-manual)
   (define-key map  "uP"  'tinydebian-url-policy-best-practises))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mode-gnus-summary-maybe-turn-on ()
  "Activate tinydebian-bts-mode if group name contains word 'Debian'"
  (when (and (boundp 'gnus-newsgroup-name)
             (stringp gnus-newsgroup-name)
             (string-match "debian" gnus-newsgroup-name))
    (turn-on-tinydebian-bts-mode)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mode-maybe-turn-on ()
  "Activate tinydebian-bts-mode if buffer contains word 'Debian'"
  (when (save-excursion
          (goto-char (point-min))
          (re-search-forward "debian" nil t))
    (turn-on-tinydebian-bts-mode)))

;;}}}
;;{{{ Install: generate severity function etc.

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-install-severity-functions ()
  "Generate `tinydebian-severity-select-*' user functions."
  ;; Generate functions on run-time.
  (mapcar
   (function
    (lambda (x)
      (let ((sym (intern (format "tinydebian-severity-select-%s"  x)))
            def)
        (setq def
              (` (defun (, sym) ()
                   "Set Severity level `tinydebian-:severity-selected'."
                   (interactive)
                   (setq  tinydebian-:severity-selected (, x)))))
        (eval def))))
   '("critical"
     "grave"
     "serious"
     "important"
     "normal"
     "minor"
     "wishlist"
     "fixed")))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-find-file-hooks ()
  "Install `font-lock-keywords' for log files."
  (tinydebian-font-lock-keywords))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-install-font-lock-keywords (&optional uninstall)
  "Install colors to all current buffers."
  (dolist (buffer (buffer-list))
    (with-current-buffer buffer
      (tinydebian-font-lock-keywords uninstall))))

;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tinydebian-install-in-buffers (&optional uninstall)
  "Install or UNINSTALL `tinydebiab-bts-mode' in existing buffers.
Activate on Gnus summary and article modes if there is word 'Debian'.
Activate on files whose path matches
`tinydebian-:install-buffer-file-name-regexp'."
  (flet ((search (regexp)
                 (save-excursion
                   (goto-char (point-min))
                   (re-search-forward regexp nil t))))
    (dolist (buffer (buffer-list))
      (let (doit)
        (with-current-buffer buffer
          (cond
           ((and (stringp buffer-file-name)
                 (string-match tinydebian-:install-buffer-file-name-regexp
                               buffer-file-name))
            (setq doit t))
           ((and (eq major-mode 'gnus-summary-mode)
                 (boundp 'gnus-newsgroup-name)
                 (string-match
                  tinydebian-:install-gnus-newsgroup-name-regexp
                  gnus-newsgroup-name))
            (setq doit t))
           ((and (eq major-mode 'gnus-article-mode)
                 (search "debian"))
            (setq doit t))
           ((search (concat
                     "bug#[0-9][0-9][0-9][0-9][0-9][0-9]\\>"
                     "\\|Closes +#[0-9][0-9][0-9][0-9][0-9][0-9]"))
            (setq doit t)))
          (if uninstall
              (turn-off-tinydebian-bts-mode)
            (turn-on-tinydebian-bts-mode)))))))

;;; ----------------------------------------------------------------------
;;;
;;;###autoload (defalias 'tinydebian-bug-report-mail 'debian-reportbug-package)
(defun tinydebian-install (&optional uninstall)
  "Install or UNINSTALL package."
  (interactive "P")
  ;;  This just hides from byte compiler fucntion definition
  ;;  so that it does not remember how amny arguments it takes
  ;;
  ;;  function tinydebian-bug-report-mail used to take 0+ arguments,
  ;;  now takes 1 function tinydebian-bug-report-mail defined multiple
  ;;  times in this file
  ;;
  (let ((sym 'tinydebian-bug-report-mail))
    (defalias sym 'debian-reportbug-package))
  (cond
   (uninstall
    ;;(remove-hook 'write-file-hooks 'tinydebian-auto-save)
    (tinydebian-install-font-lock-keywords 'uninstall)
    (remove-hook 'find-file-hooks 'tinydebian-find-file-hooks)
    (remove-hook 'gnus-summary-prepare-hook
                 'tinydebian-bts-mode-gnus-summary-maybe-turn-on)
    (remove-hook 'gnus-article-prepare-hook
                 'tinydebian-bts-mode-maybe-turn-on)
    (tinydebian-install-in-buffers 'uninstall))
   (t
    ;; (add-hook 'write-file-hooks 'tinydebian-auto-save)
    (tinydebian-install-font-lock-keywords)
    (add-hook 'find-file-hooks  'tinydebian-find-file-hooks)
    (add-hook 'gnus-summary-prepare-hook
              'tinydebian-bts-mode-gnus-summary-maybe-turn-on)
    (add-hook 'gnus-article-prepare-hook
              'tinydebian-bts-mode-maybe-turn-on)
    (tinydebian-install-in-buffers)))
  nil)

;;}}}
;;{{{ Utility functions

;;; ----------------------------------------------------------------------
;;;
(put 'tinydebian-with-buffer-macro 'edebug-form-spec '(body))
(put 'tinydebian-with-buffer-macro 'lisp-indent-function 1)
(defmacro tinydebian-with-buffer-macro (buffer &rest body)
  "Create BUFFER, empty it and run BODY.
Variable `buffer' is available in this macro."
  `(let ((buffer (get-buffer-create ,buffer)))
     (with-current-buffer buffer
       (erase-buffer)
       ,@body)))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-buffer-match-string (regexp &optional start)
  "Search REGEX at optional START point and return submatch 1."
  (save-excursion
    (if start
        (goto-char start))
    (if (re-search-forward regexp nil t)
        (match-string 1))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-call-process (prg &optional buffer &rest args)
  "Call PRG with list of ARGS and print output to current buffer or BUFFER."
  (apply 'call-process
         prg
         (not 'infile)
         (or buffer (current-buffer))
         (not 'real-time-display)
         args))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-packages-browse-url-compose
  (keyword &optional search-on distribution section)
  "Return URL search string.
Argument: KEYWORD
Optional: SEARCH-ON DISTRIBUTION SECTION."
  (format (concat tinydebian-:url-http-package-search
                  "keywords=%s&"
                  "searchon=%s&"
                  "subword=1&"
                  "version=%s&"
                  "release=%s")
          keyword
          (or search-on    "names")
          (or distribution "all")
          (or section      "all")))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-string-delete-newlines (string)
  "Delete newlines from STRING."
  (ti::string-regexp-delete "[\r\n]" string))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-font-lock-keywords (&optional uninstall)
  "Add color support to various log files by setting
`font-lock-keywords'."
  (interactive)
  (let* ((today  (ti::date-standard-rfc-regexp "mon-date"))
         ;; (cs     (or comment-start-skip "[ \t]+"))
         (file   "")
         keywords)
    (when (stringp buffer-file-name)
      (setq file (or buffer-file-name "no-name?")))
    (setq
     keywords
     (cond
      ;; ............................................. Linux log files ...
      ;; /var/log/
      ((string-match "/log/messages$" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list (concat today " +[0-9]+:+[0-9]+:+[0-9]+")
                    0 'font-lock-function-name-face)
              (list
               (concat
                "^... +[0-9]+ +[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              (list
               (concat "restarted\\|started"
                       "\\|ignoring"
                       "\\|Linux version.*")
               0 'font-lock-comment-face))))

      ((string-match "mail\\.log\\|mail\\.info" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list (concat today " +[0-9]+:+[0-9]+:+[0-9]+")
                    0 'font-lock-function-name-face)
              (list
               (concat
                "^... +[0-9]+ ++[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              '("timed out\\|did not.*"
                0 tinydebian-:warn-face)
              (list
               (concat "\\(from\\|to\\)=\\([^ ,\t\r\n]+\\)")
               2 'font-lock-comment-face))))


      ((string-match "daemon\\.log" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list
               (concat
                "^... +[0-9]+ +[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              (list
               (concat "connection attempt" ;;  See "iplogger" package
                       )
               0 'tinydebian-:warn-face)
              (list
               (concat "signal +[0-9]+\\|no such user"
                       "\\|connect from .*")
               0 'font-lock-comment-face))))


      ((string-match "auth\\.log" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list (concat today " +[0-9]+:+[0-9]+:+[0-9]+")
                    0 'font-lock-function-name-face)
              (list
               (concat
                "^... +[0-9]+ +[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              (list
               (concat "opened +for +[^ \t\r\n]+")
               0 'tinydebian-:warn-face)
              '( "for user \\(root\\)"
                 1 font-lock-string-face)
              '( "from \\([^ \t\r\n]+\\)"
                 1 font-lock-type-face)
              '( "for +\\([^ \t\r\n]+\\) +from"
                 1 font-lock-comment-face)
              '( "for user +\\([^ \t\r\n]+\\)"
                 1 font-lock-comment-face))))

      ((string-match "syslog" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list (concat today " +[0-9]+:+[0-9]+:+[0-9]+")
                    0 'font-lock-function-name-face)
              (list
               (concat
                "^... +[0-9]+ +[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              (list
               (concat "Invalid.*"
                       ;; portmap[135]: cannot bind udp: Address already in use
                       "\\|cannot"
                       "\\|Connection timed out"
                       ;;  See iplogger(1)
                       "\\|connection attempt"
                       ;;  See portsentry(1)
                       "\\|attackalert:.* +to +.*port.*"
                       ;;  apm -s failed
                       "\\| failed"
                       "\\|did not .*")
               0 'tinydebian-:warn-face)
              '("to=\\([^ \t\r\n]+\\)"
                1 font-lock-comment-face)
              '("(\\([^ )\t\r\n]+\\)) CMD "
                1 font-lock-comment-face)
              '("CMD .*"
                0 font-lock-constant-face)
              '("inetd"2
                0 font-lock-type-face)
              (list
               (concat
                "program exit.*\\|.*started.*"
                ;;  btpd daemon
                "\\|synchronisation lost"
                )
                0 font-lock-keyword-face))))

      ))
    (when keywords
      (cond
       (uninstall
         (setq font-lock-keywords nil))
       ((or font-lock-mode
            global-font-lock-mode
            (font-lock-mode-maybe 1))
        (setq font-lock-keywords keywords))))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-string-parse-wnpp-alert (str)
  "Parse wnpp-alert(1) like line. Return '(bug package bug-type desc)
  RFA 321654 debtags -- Enables support for package tags."
  (let (case-fold-search)
    (when (string-match
           (concat
            "\\<\\(RF.\\|IT.\\|O\\) +\\([0-9]+\\) +"
            "\\([^ \t\r\n]+\\) +-- +\\(.+[^ \t\r\n]\\)")
           str)
      (list
       (match-string 2 str)
       (match-string 3 str)
       (match-string 1 str)
       (match-string 4 str)))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-nbr-string (str)
  "Read bug nbr from STR."
  (or (and (string-match "#\\([0-9]+\\)" str)
           (match-string 1 str))
      (multiple-value-bind (bug)
          (tinydebian-bug-string-parse-wnpp-alert str)
        bug)
      ;;   NNNN@bugs.debian.org
      (and (string-match (concat "\\([0-9]+\\)\\(?:-[a-z]+\\)?@"
                                 tinydebian-:bts-email-address)
                         str)
           (match-string 1 str))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-nbr-at-current-point ()
  "Read bug number from current point"
  (let ((table (syntax-table))
        word)
    (with-syntax-table table
      (modify-syntax-entry ?# "w" table)
      (tinydebian-bug-nbr-string (current-word)))))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bug-nbr-current-line ()
  "Read bug number from current line"
  (let* ((line (buffer-substring-no-properties
                (line-beginning-position)
                (line-end-position))))
    (tinydebian-bug-nbr-string line)))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bug-nbr-forward (&optional regexp)
  "Read bug#NNNN from current point forward.
If optional REGEXP is sebt, it must take number in submatch 1."
  (tinydebian-buffer-match-string (or regexp "Bug#\\([0-9]+\\)")))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bug-hash-forward ()
  "Search #NNNN forward."
  (tinydebian-bug-nbr-forward "#\\([0-9]+\\)"))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bug-nbr-buffer (&optional regexp)
  "Read bug#NNNN or REGEXP from buffer."
  (save-excursion
    (goto-char (point-min))
    (tinydebian-bug-nbr-forward)))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bug-hash-buffer ()
  "Search #NNNN from buffer."
  (tinydebian-bug-nbr-buffer "#\\([0-9]+\\)"))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-email-cc-to-bug-nbr ()
  "Read BTS number from CC or To"
  (let* ((str (mail-fetch-field "To")))
    (or (and str
             (tinydebian-bug-nbr-string str))
        (and (setq str (mail-fetch-field "Cc"))
             (tinydebian-bug-nbr-string str)))))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-email-subject-bug-nbr ()
  "Read BTS number from Subject"
  (let* ((subject (mail-fetch-field "Subject")))
    (and subject
         (tinydebian-bug-nbr-string subject))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-nbr-any (&rest args)
  "Try various methods to find bug tracking number. Ignore ARGS.
At current point, current line, headers of the mail message
(CC, To, Subject), forward from point, whole buffer."
  (or (tinydebian-bug-nbr-at-current-point)
      (tinydebian-bug-nbr-current-line)
      (tinydebian-email-cc-to-bug-nbr)
      (tinydebian-email-subject-bug-nbr)
      (tinydebian-bug-nbr-forward)
      (tinydebian-bug-nbr-buffer)
      (tinydebian-bug-hash-forward)
      (tinydebian-bug-hash-buffer)))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bug-nbr-search ()
  "Call hook `tinydebian-:find-bug-nbr-hook' until value returned."
  (run-hook-with-args-until-success 'tinydebian-:find-bug-nbr-hook))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-package-name-header-pool ()
  "Search Filename: pool/main/p/<package>."
  (tinydebian-buffer-match-string
   "^Filename: pool.*/\\([^/ \t\r\n]+\\)/"
   (point-min)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-package-name-header-package ()
  "Search Package: <package>."
  (tinydebian-buffer-match-string
   "^Package: +\\([^/ \t\r\n]+\\)/"
   (point-min)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-parse-string-with-bug (str)
  "Return '(bug type package description) for common matches."
  (let (bug
        type
        package
        desc
        case-fold-search)
    (cond
     ((string-match "\\<\\([A-Z][A-Z][A-Z]\\|O\\): *\\(.*\\)" str)
      (setq type (match-string 1 str)
            desc (match-string 2 str)
            bug  (tinydebian-bug-nbr-string str))
      (when (string-match "^\\([a-z].+\\) +--+ *\\(.*\\)" desc)
        (setq package (match-string 1 desc)
              desc    (match-string 2 desc))))
     ((string-match "Bug#\\([0-9]+\\): *\\(.*\\)" str)
      (setq bug  (match-string 1 str)
            desc (match-string 2 str))))
    (list bug type package desc)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-parse-string-with-package (str)
  "Return '(package description) for common matches."
  (let (case-fold-search)
    (cond
     ((string-match
       "[fF]ixed in\\(?: NMU of\\)? \\([a-z][^ \t\r\n]+\\) +\\(.*\\)" str)
      (list (match-string 1 str)
            str))
     ((string-match "^\\([a-z][a-z0-9-]+\\): +\\(.*\\)" str)
      (list (match-string 1 str)
            (match-string 2 str))))))

;;; ----------------------------------------------------------------------
;;; (tinydebian-bts-parse-string-1 "Bug#353353: RFP: appweb -- very ...")
;;; (tinydebian-bts-parse-string-1 "Bug#352429: marked as done (ITA: cdrdao  -- records CDs in Disk-At-Once (DAO) mode)")
;;; (tinydebian-bts-parse-string-1 "Bug#351502: fixed in nvu 1.0final-1")
;;; (tinydebian-bts-parse-string-1 "Bug#352533: Fixed in NMU of sa-exim 4.2-3")
;;; (tinydebian-bts-parse-string-1 "Bug#244582: UFO:AI is back")
;;; (tinydebian-bts-parse-string-1 "")
;;; (tinydebian-bts-parse-string-1 "")
(defun tinydebian-bts-parse-string-1 (str)
  "Parse STR and Return '(bug type package description)."
  (when (stringp str)
    ;;  Treat long "folded" subject like:
    ;;
    ;;  Subject: Bug#353588 acknowledged by developer (Re: Bug#353588: lintian:
    ;;     [add new rule] check debian/control::Description better ...
    ;;
    (setq str
          (replace-regexp-in-string "[\r\n]+" " " str))
    (multiple-value-bind (bug type package desc)
        (tinydebian-bts-parse-string-with-bug str)
      (when (and (not package)
                 desc)
        (multiple-value-bind (ret-pkg ret-desc)
            (tinydebian-bts-parse-string-with-package desc)
          (setq package ret-pkg
                desc    ret-desc)))
      (if (and (stringp desc)
               (string= desc ""))
          (setq desc nil))
      (if (and bug desc)
          (list bug type package desc)))))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bts-parse-string-current-line ()
  (let ((str (buffer-substring-no-properties
              (line-beginning-position)
              (line-end-position))))
    (tinydebian-bts-parse-string-1 str)))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bts-parse-string-subject ()
  (let ((str (mail-fetch-field "Subject")))
    (when str
      (tinydebian-bts-parse-string-1 str))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-package-name-current-line ()
  (let* ((line (buffer-substring-no-properties
                (line-beginning-position)
                (line-end-position))))
    (when line
      (multiple-value-bind (bug package)
          (tinydebian-bug-string-parse-wnpp-alert line)
       package))))

;;; ----------------------------------------------------------------------
;;;
(defun my-debian-bug-package-name-any ()
  "Search package name."
  (or (tinydebian-bug-package-name-current-line)
      (tinydebian-bug-package-name-header-pool)
      (tinydebian-bug-package-name-header-package)
      (progn
        (multiple-value-bind (bug type-orig package description)
            (tinydebian-bts-parse-string-subject)
          package))))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-email-subject-type-parse ()
  "Read BTS Subject and return '(TYPE SUBJECT)"
  (let* ((subject (mail-fetch-field "Subject")))
    (when subject
      ;;  Bug#292579: marked as done (RFP: miwm -- MIcroscopic Window
      (let (type subject bug)
        (when (string-match "\\(?: (?\\)\\([a-z]+\\):\\(.*\\)" subject)
          (setq type    (match-string 1 subject)
                subject (match-string 2 subject)))
        (setq bug
              (tinydebian-bug-nbr-string subject))
        (list type subject bug)))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-browse-url-browse-url (url &rest args)
  "Call `browse-url' and ignore ARGS."
  (browse-url url))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-browse-url-lynx-dump (url &optional mode)
  "Run lynx(1) with option -dump using URL.
Optional MODE is hint to activate `tinydebian-bts-mode' on text buffer"
  (let* ((path (executable-find "lynx")))
    (if (not path)
        (error "TinyDebian: [ERROR] `lynx' not found for %s" url)
      (tinydebian-with-buffer-macro tinydebian-:buffer-www
        (message "TinyDebian: Wait, accessing %s" url)
        (tinydebian-call-process path nil "-dump" url)
        (if mode
            (turn-on-tinydebian-bts-mode))
        (goto-char (point-min))
        (display-buffer (current-buffer))))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-browse-url-1 (url &optional mode)
  "Call `tinydebian-:browse-url-function' with URL.
Optional MODE is hint to activate `tinydebian-bts-mode' on text buffer"
  (if tinydebian-:browse-url-function
      (funcall tinydebian-:browse-url-function url mode)
    (tinydebian-browse-url-browse-url url)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-browse-url-by-bug (bug)
  "Jump to BUG report."
  (interactive
   (list (read-string "Browse URL by bug number: "
                      (tinydebian-bug-nbr-search))))
  (tinydebian-browse-url-1
   (format "http://bugs.debian.org/%s"
           (if (numberp bug)
               (int-to-string bug)
             bug))
   bug))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-browse-url-by-package-name (package)
  "Jump to PACKAGE description."
  (interactive
   (list (read-string "Browse desription URL by package name: "
                      (my-debian-bug-package-name-any))))
  (tinydebian-browse-url-1
   (tinydebian-packages-browse-url-compose package)
   package))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-browse-url-by-package-bugs (package)
  "Jump to PACKAGE description."
  (interactive
   (list (read-string "Browse bugs URL by package name: "
                      (my-debian-bug-package-name-any))))
  (tinydebian-browse-url-1
   (format "%s%s"
           tinydebian-:url-http-package-bugs
           package)
   package))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-command-show-wnpp-alert ()
  "Check for installed packages up for adoption or orphaned.
Requires that program wnpp-alert(1) has been installed."
  (interactive)
  (let* ((bin  "wnpp-alert")
         (path (executable-find bin)))
    (cond
     ((not bin)
      (message "TinyDebian: [ERROR] program `%s' is not installed."
               bin))
     (t
      (tinydebian-with-buffer-macro tinydebian-:buffer-wnpp-alert
        (message "TinyDebian: wait, running %s..." path)
        (tinydebian-call-process path)
        (message "TinyDebian: wait, running %s... Done." path)
        (goto-char (point-min))
        (turn-on-tinydebian-bts-mode)
        (display-buffer buffer))))))

;;}}}
;;{{{ BTS URL pages

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-url-page-compose (page-type)
  "Return URL to page."
  (let ((page ((assoc page-type tinydebian-:url-debian-page-alist))))
    (unless page
      (error "TinyDebian: unknow page-typpe `%s'" page-type))
    (cdr page)))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-url-debian-browse-url (page-type)
  "Browse Debian WWW pages."
  (tinydebian-browse-url-1 (tinydebian-url-page-compose page-type)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-url-bts-ctrl-page ()
  "Browse BTS control page."
  (interactive)
  (tinydebian-url-debian-browse-url 'bts-control))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-url-policy-manual ()
  "Browse policy manual page."
  (interactive)
  (tinydebian-url-debian-browse-url 'policy))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-url-policy-best-practises ()
  "Browse policy manual page: best practises section."
  (interactive)
  (tinydebian-url-debian-browse-url 'best-practises))

;;}}}
;;{{{ WNPP URLs

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-url-wnpp-compose (page-type)
  "Return URL to search"
  (let ((page (assoc page-type tinydebian-:url-http-wnpp-page-alist)))
    (unless page
      (error "TinyDebian: unknow page-typpe `%s'" page-type))
    (format "%s/%s" tinydebian-:url-http-wnpp-page-main (cdr page))))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-url-wnpp-browse-url (page-type)
  "Browse WNPP PAGE-TYPE."
  (tinydebian-browse-url-1 (tinydebian-url-wnpp-compose page-type)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-url-wnpp-itp ()
  "Browse WNPP ITP page."
  (interactive)
  (tinydebian-url-wnpp-browse-url "ITP"))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-url-wnpp-rfp ()
  "Browse WNPP RFP page."
  (interactive)
  (tinydebian-url-wnpp-browse-url "RFP"))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-url-wnpp-rfh ()
  "Browse WNPP RFH page."
  (interactive)
  (tinydebian-url-wnpp-browse-url "RFH"))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-url-wnpp-rfa ()
  "Browse WNPP RFA page."
  (interactive)
  (tinydebian-url-wnpp-browse-url "RFA"))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-url-wnpp-orphaned ()
  "Browse WNPP orphaned page."
  (interactive)
  (tinydebian-url-wnpp-browse-url "O"))

;;}}}
;;{{{ BTS functions: Debian Developer interface to bug tracking system

;;; ----------------------------------------------------------------------
;;;
(put 'tinydebian-bts-mail-compose-macro 'edebug-form-spec '(body))
(put 'tinydebian-bts-mail-compose-macro 'lisp-indent-function 5)
(defmacro tinydebian-bts-mail-compose-macro
  (bug type package subject email &rest body)
  "Compose mail with SUBJECT and run BODY."
  (let ((name (gensym "name-")))
    `(let ((,name (format "*Mail Debian BTS %s*"
                          (cond
                           ((and ,bug ,type ,package)
                            (format "%s %s %s"
                                    ,type ,package ,bug))
                           ((and ,bug ,package)
                            (format "%s %s"
                                    ,package ,bug))
                           (t
                            (or ,bug
                                ,subject
                                ""))))))
       (pop-to-buffer (get-buffer-create ,name))
       (erase-buffer)
       (mail-setup
        (if ,email
            ,email
          (tinydebian-bts-email-compose "control"))
        ,subject
        nil
        nil
        nil
        nil)
       (cond
        ((or (featurep 'message)
             (eq mail-user-agent 'message-user-agent))
         (message-mode))
        (t
         (mail-mode)))
       ,@body)))

;;; ----------------------------------------------------------------------
;;;
(put 'tinydebian-bts-mail-type-macro 'edebug-form-spec '(body))
(put 'tinydebian-bts-mail-type-macro 'lisp-indent-function 4)
(defmacro tinydebian-bts-mail-type-macro (type pkg email subject &rest body)
  "Compose a TYPE request and run BODY.
Variables available: bugnbr, type-orig, package, subject; but these
can all be nil."
  (let ((subj (gensym "subject-")))
    `(multiple-value-bind (bugnbr type-orig package description)
         (or (tinydebian-bts-parse-string-current-line)
             (tinydebian-bts-parse-string-subject))
       (if (stringp ,pkg) ;; Use input argument
           (setq package ,pkg))
       (let ((,subj (or ,subject
                        (if ,type
                            (format "%s: %s%s"
                                    ,type
                                    (if package
                                        (format "%s -- " package)
                                      "")
                                    (or description ""))
                          ""))))
         (tinydebian-bts-mail-compose-macro
             bugnbr
             ,type
             package
             ,subj
             ,email
           (goto-char (point-max))
           ,@body)))))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bts-mail-ask-bug-number (&optional type)
  "Ask bug number. Return as '(bug) suitable for interactive"
  (read-string
   (format "Debian BTS %sbug number: "
           (if type
               (concat type " ")
             ""))
   (tinydebian-bug-nbr-any)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mail-type-ita (bug)
  "Send an ITA request."
  (interactive (list (tinydebian-bts-mail-ask-bug-number "ITA")))
  (tinydebian-bts-mail-type-macro "ITA" nil nil nil
    (insert
     (format "\
package wnpp
retitle %s %s
owner %s !
thanks
"
             bug
             subject
             bug))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mail-type-rfp (package license homepage desc)
  "Send an ITP request."
  (interactive
   (let* ((name    (read-string
                    "RFP package name [required; lowercase]: "))
          (desc    (read-string
                    "Package description [required]: "))
          (license (completing-read
                    "License [required]: "
                    (mapcar (lambda (x)
                              (cons x 1))
                            tinydebian-:wnpp-template-licences-alist)))
          (url     (read-string
                    "Project homepage URL [required]: ")))
     (list name license url desc)))
  (flet ((replace (regexp str &optional point)
                  (when (and (stringp str)
                             (not (string= "" str)))
                    (goto-char (or point
                                   (point-min)))
                    (when (re-search-forward regexp nil t)
                      (replace-match str 'literal nil nil 1)))))
    (let ((arg-pkg package)) ;; Due to macro reserving `package'.
      (tinydebian-bts-mail-type-macro "ITP" arg-pkg nil nil
        (insert tinydebian-:-wnpp-template-new-package)
        (replace "\\(<package>.*\\)"    package)
        (replace "\\(<homepage:.*\\)"   homepage)
        (replace "\\(<license:.*\\)"    license)
        (replace "\\(<short desc>.*\\)" desc)
        (mail-position-on-field "Subject")
        (beginning-of-line)
        (replace ": \\(.*\\)"
                 (format "RFP: %s -- %s" package desc)
                 (point))
        (goto-char (point-max))))))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mail-ctrl-severity (bug severity)
  "Compose BTS control message: severity."
  (interactive
   (list (tinydebian-bts-mail-ask-bug-number)
         (completing-read
          "BTS severity: "
          tinydebian-:severity-list
          nil
          'match)))
  (tinydebian-bts-mail-type-macro
      nil nil nil
      (format "Change of severity -- %s" severity)
    (insert
     (format "\
severity %s %s
thanks

"
             bug
             severity))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mail-ctrl-tags (bug tag-string)
  "Compose BTS control message: tags."
  (interactive
   (let ((bug (tinydebian-bts-mail-ask-bug-number))
         tag
         list)
     (while (or (null tag)
                (not (string= "" tag)))
       (setq tag (completing-read
                  "BTS tag [RET when done]: "
                  tinydebian-:tags-list
                  nil
                  'match))
       (unless (string= "" tag)
         (push tag list)))
     (list bug
           (mapconcat 'concat list " "))))
  (tinydebian-bts-mail-type-macro
      nil nil nil
      (format "Change of tags -- %s" tag-string)
    (insert
     (format "\
tags %s + %s
thanks

"
             bug
             tag-string))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mail-ctrl-reassign (bug &optional package)
  "Compose BTS control message: reassign."
  (interactive
   (list (tinydebian-bts-mail-ask-bug-number)
         (read-string "Reassign to package: ")))
  (tinydebian-bts-mail-type-macro
      nil nil nil
      (format "Reassign Bug#%s" bug)
    (insert
     (format "\
reassign %s %s
thanks

"
             bug
             (if (and package
                      (not (string= "" package)))
                 package
               "<to-package>")))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mail-ctrl-retitle (bug title)
  "Compose BTS control message: reassign."
  (interactive
   (list (tinydebian-bts-mail-ask-bug-number)
         (read-string "New title: ")))
  (tinydebian-bts-mail-type-macro
      nil nil nil
      (format "Reassign Bug#%s" bug)
    (insert
     (format "\
retitle %s %s
thanks

"
             bug
             title))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mail-ctrl-close (bug)
  "Compose BTS control message: close."
  (interactive (list (tinydebian-bts-mail-ask-bug-number)))
  (let* ((email (tinydebian-bts-email-compose (format "%s-done" bug))))
    (tinydebian-bts-mail-type-macro
        nil nil email
        (format "Close #%s" bug)
      (insert
       (format "\
Version:

<explain reason here>
"
               )))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bts-mail-ctrl-forward (bug)
  "Compose BTS control message: forward bug report to upstream."
  (interactive
   (list (tinydebian-bts-mail-ask-bug-number))
  (let* ((email (tinydebian-bts-email-compose
                 (format "%s-forwarded" bug))))
    (tinydebian-bts-mail-type-macro
        nil nil email
      (format "Debian Bug#%s -- forwarded upstream" bug)
      (mail-position-on-field "Cc")
      (insert email)
      (insert
       (format "\
[Please keep the CC]

"
               ))))))


;;}}}
;;{{{ Dpkg, apt functions

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-read-field-content-1 ()
  "Read content. Point must be positionioned at Field:-!-."
  (let* ((str (if (looking-at " +\\(.*\\)")
                  (match-string 1))))
    (while (and (not (eobp))
                (zerop (forward-line 1))  ;; Did it
                (looking-at "^\\( +.*\\)"))
      (setq str (concat (or str "") (match-string 1))))
    str))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-read-field-content (&optional field)
  "Read FIELD forward. FIELD ust be name like `Package'.
Be sure to call `tinydebian-package-narrow-to-region' first."
  (when (re-search-forward (format "^%s:" field) nil t)
    (tinydebian-package-read-field-content-1)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-parse-info-all ()
  "Parse all fields forward. Return '((field . info) (field . info) ..)."
  (let* (field
         alist)
    (while (re-search-forward "^\\([^ \t\r\n]+\\):" nil t)
      (setq field (match-string 1))
      (push (cons field (tinydebian-package-read-field-content-1))
            alist))
    (nreverse alist)))

;;; ----------------------------------------------------------------------
;;;
(defmacro tinydebian-bts-email-compose (address)
  "Send message to Debian BTS at ADDRESS."
  `(format "%s@%s" ,address tinydebian-:bts-email-address))

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-bts-email-control ()
  (tinydebian-bts-email-compose "control"))

;;; ----------------------------------------------------------------------
;;;
(defmacro tinydebian-package-narrow-to-region (&rest body)
  "Search dpkg -s result from current point forward and narrow around it.
Point is put at the beginning of region.
Variable `package' contains the package name."
  (`
   (let* (beg-narrow
          package)
     (when (re-search-forward "^Package: +\\([^ \t\r\n]+\\) *$" nil t)
       (setq beg-narrow (line-beginning-position))
       (setq package (match-string 1))
       (when (re-search-forward "^[ \t]*$" nil t)
         (ti::narrow-safe beg-narrow (point)
           (ti::pmin)
           (,@ body)))))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-status-parse-depends-1 ()
  "Parse `Depends' field content from current point forward.
There must nothing else in the buffer."
  (let* (name
         op
         ver
         list)
    (while (re-search-forward "\\([a-z][^ ,()\t\r\n]+\\)" nil t)
      (setq name (ti::remove-properties (match-string 1))
            op   nil
            ver  nil)
      (cond
       ((looking-at " +(\\([=><]+\\) +\\([^ ,()\t\r\n]+\\))")
        (setq op   (ti::remove-properties (match-string 1))
              ver  (ti::remove-properties (match-string 2))))
       ((looking-at " *,?")))
      (goto-char (match-end 0))
      (push (list name op ver) list))
    (nreverse list)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-status-parse-depends (depends)
  "Parse `Depends' field from DEPENDS string.
Example of the DEPENDS string:

    \"libc6 (>= 2.2.4-2), cron (>= 3.0pl1-42)\"

Returned list is

   '((\"libc6\" \">=\" \"2.2.4-2\")
     (\"cron\"  \">=\" \"3.0pl1-42\"))."
   (with-temp-buffer
      (insert depends)
      (ti::pmin)
      (tinydebian-package-status-parse-depends-1)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-status-apt-file (package)
  "Use apt-file PACKAGE (must be installed separately) to find upstream."
  (let* ((bin (executable-find "apt-file")))
    (cond
     ((null bin)
      (message "TinyDebian: no `apt-fil' found along PATH (emacs `exec-path').")
      (message "TinyDebian: Please run 'apt-get install apt-file'")
      nil)
     (t
      ;; #todo:
      ))))

;;; ----------------------------------------------------------------------
;;;
;;; Package: autolog
;;; Status: install ok installed
;;; Priority: extra
;;; Section: admin
;;; Installed-Size: 45
;;; Maintainer: Nicols Lichtmaier <nick@debian.org>
;;; Version: 0.35-10
;;; Depends: libc6 (>= 2.2.4-2), cron (>= 3.0pl1-42)
;;; Recommends: mail-transport-agent
;;; Conffiles:
;;;  /etc/autolog.conf a3fcae584ed74543a4a943e722593ff6
;;;  /etc/cron.d/autolog 805d268ea44c645299defc1c14495282
;;; Description: Terminates connections for idle users
;;;  Autolog terminates connections considered to be idle based on a large
;;;  variety of parameters.
;;;
(defun tinydebian-package-status-dpkg-s (package)
  "Consult dpkg -s PACKAGE"
  (let* ((dpkg  tinydebian-:bin-dpkg))
    (cond
     ((not dpkg)
      (message "TinyDebian: no `dpkg' found along PATH (emacs `exec-path').")
      nil)
     (t
      (with-temp-buffer
        (message "TinyDebian: Running ... dpkg -s %s" package)
        (tinydebian-call-process dpkg nil (list "-s" package))
        (ti::pmin)
        (when (re-search-forward "^Use dpkg" nil t)
          (message "TinyDebian: `dpkg`-s %s' returned error [%s]"
                   package
                   (buffer-string)))
        (tinydebian-package-parse-info-all))))))


;;; ----------------------------------------------------------------------
;;; dpkg -S dh_make
;;;
;;; debhelper: /usr/bin/dh_makeshlibs
;;; dh-make: /usr/share/debhelper/dh_make/debian/postrm.ex
;;; dh-make: /usr/share/debhelper/dh_make/native
;;; dh-make: /usr/share/debhelper/dh_make/debian/changelog
;;; dh-make: /usr/share/debhelper/dh_make/debianl/shlibs.local.ex
;;; dh-make: /usr/share/man/man1/dh_make.1.gz
;;; dh-make: /usr/bin/dh_make
;;; dh-make: /usr/share/debhelper/dh_make/debiank/README.Debian
;;; dh-make: /usr/share/debhelper/dh_make/debianm/control
;;; dh-make: /usr/share/debhelper/dh_make/debian/init.d.ex
;;; dh-make: /usr/share/debhelper/dh_make/debian/cron.d.ex
;;; dh-make: /usr/share/debhelper/dh_make/debianm/rules
;;; dh-make: /usr/share/debhelper/dh_make/licenses/lgpl
;;; dh-make: /usr/share/debhelper/dh_make/debiank/control
;;; dh-make: /usr/share/debhelper/dh_make/debians/rules
;;; dh-make: /usr/share/debhelper/dh_make/debianl/package1.dirs
;;; dh-make: /usr/share/debhelper/dh_make/native/changelog
;;; dh-make: /usr/share/debhelper/dh_make/licenses/bsd
;;; dh-make: /usr/share/debhelper/dh_make/debianm/package-doc.files
;;; dh-make: /usr/share/debhelper/dh_make/debians/watch.ex
;;; dh-make: /usr/share/debhelper/dh_make/licenses/gpl
;;; dh-make: /usr/share/debhelper/dh_make/licenses/blank
;;;
(defun tinydebian-package-status-dpkg-S-parse (package)
  "Examine dpkg -S PACKAGE listing and return upstream package name."
  (ti::pmin)
  (when (re-search-forward (concat ":.*/\\([a-z]+\\)/"
                                   package
                                   "[/ \t\r\r]"
                                   )
                           nil t)
    (match-string 1)))

;;; ----------------------------------------------------------------------
;;;
;;;
(defun tinydebian-package-status-dpkg-S (file)
  "Consult dpkg -S FILE
In this case, the package is unknown and the upstream package is
searched. If upstream package is found, call
`tinydebian-package-status-dpkg-s'. "
  (let* ((dpkg  tinydebian-:bin-dpkg))
    (cond
     ((not dpkg)
      (message "TinyDebian: no `dpkg' found along PATH (emacs `exec-path').")
      nil)
     (t
      (with-temp-buffer
        (message "TinyDebian: Running ... dpkg -S %s" file)
        (tinydebian-call-process dpkg nil (list "-S" file))
        (let ((upstream (tinydebian-package-status-dpkg-S-parse file)))
          (cond
           ((null upstream)
            (message "TinyDebian: dpkg -S doesn't know upstream package for `%s'"
                     file)
            nil)
           (t
            (tinydebian-package-status-dpkg-s upstream)))))))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-wnpp-main-interactive ()
  "Ask the type of request for WNPP package.
References:
  `tinydebian-:menu-wnpp'
  `tinydebian-:menu-wnpp-selected'"
  (setq tinydebian-:menu-wnpp-selected nil)
  (ti::menu-menu 'tinydebian-:menu-wnpp)
  tinydebian-:menu-wnpp-selected)

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-buffer-ask-input (message buffer &optional clear)
  "Write MESSAGE to the buffer ans ask user to type input.
The MESSAGE should contgain properly formatted text."
  (let* ((buffer (ti::temp-buffer buffer clear)))
    ;; (switch-to-buffer buffer)
    ;; #todo:
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-wnpp-main (request-type)
  "Submit REQUEST-TYPE against WNPP pseudo package.
WNPP is used for requesting to be a new Debian maintainer and
for taking maintenance of other packages. Refer to
http://www.debian.org/devel/wnpp and
http://www.debian.org/doc/packaging-manuals/developers-reference/ch-pkgs.en.html
and topic \"5.1 New Packages\"

REQUEST-TYPE can be symbol:

  'package 'orphan 'adopt or 'new.
  See http://www.debian.org/devel/wnpp for more information

References:

  `tinydebian-:menu-wnpp'."
  (interactive (list (tinydebian-package-wnpp-main-interactive)))
  (cond
   ((eq request-type 'package)
    )
   ((eq request-type 'new)
    (let ((name  (read-string "Proposed package name: ")))
      ;;  Checking status database

      nil))
   ((eq request-type 'orphan)
    )
   ((eq request-type 'adopt)
    )
   ((eq request-type 'new)
    )
   (t
    ;;  Nothing to do
    nil)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-status-main (package)
  "Find out PACKAGE details."
  (or (tinydebian-package-status-dpkg-s package)
      (tinydebian-package-status-dpkg-S package)
      (tinydebian-package-status-apt-file package)
      (if (string-match "^wnpp" package)
          (error "\
TinyDebian: package WNPP is special. Use tinydebian-package-wnpp-main instead."))
      (error "Tinydebian: Can't find package information. `%s'" package)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-info-from-buffer (buffer)
  "Parse dpkg -s from BUFFER. Tge buffer must contain nothing else."
  (with-current-buffer buffer
    (ti::pmin)
    (tinydebian-package-parse-info-all)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-info (&optional package prompt)
  "Get PACKAGE information. See`tinydebian-package-status'.
If PACKAGE is nil and `tinydebian-:bin-dpkg' is not available,
ask with PROMPT."
    (let* ((dpkg  tinydebian-:bin-dpkg))
      (or package
          (setq package (read-string
                         (or prompt
                             "[TinyDebian] Package name: "))))
      (or (and dpkg
               (tinydebian-package-status-main package))
          (tinydebian-package-info-from-user package prompt))))

;;}}}
;;{{{ Bug reporting interface

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-general ()
  "Return relevant system information."
  (interactive)
  (let* (
         )
    "" ;; #todo:
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-depends (info &optional depend-key)
  "Return additional Dependency INFO from item `Depends'.
DEPEND-KEY can be \"Depends\" or \"Pre-Depends\".

Example:

  Versions of packages autolog depends on:
  ii  cron            3.0pl1-72  management of regular background p
  ii  libc6           2.2.5-3    GNU C Library: Shared libraries an."
  (let* ((depends (cdr-safe (assoc
                             (or depend-key "Depends")
                             info)))
         str)
    (when depends
      (setq str "")
      (dolist (dep-info
               (tinydebian-package-status-parse-depends depends))
        (multiple-value-bind (package op version)
            dep-info

          ;; Not used yet, quiet byte compiler
          (if op
              (setq op op))
          (if version
              (setq version version))

          (let* (info2
                 desc
                 ver)
            (setq info2
                  (tinydebian-package-info
                   package
                   (format "\
\[TinyDebian] Depend. Insert `dpkg -s %s' to *scratch* and press RET: "
                           package)))
            (setq ver  (cdr-safe (assoc "Version" info2)))
            ;; cut first few characters
            (when (setq desc (cdr-safe (assoc "Description" info2)))
              (setq desc (ti::string-left desc 45)))
            (setq str
                  (concat
                   str
                   (format "%-15s %-15s %s\n" package ver desc)))))))
    str))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-os-architecture ()
  "Read architecture."
  (if (not tinydebian-:bin-dpkg)
      ""
    (with-temp-buffer
    (tinydebian-call-process
     tinydebian-:bin-dpkg  nil '("--print-installation-architecture"))
    (tinydebian-string-delete-newlines
     (buffer-string)))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-os-version ()
  "Read Debian version number."
  (let* ((file  "/etc/debian_version")
         (ret    (format "%s not found or readable." file)))
    (when (and (file-exists-p   file)
               (file-readable-p file))
      (with-temp-buffer
        (insert-file-contents-literally file)
        (setq ret
              (tinydebian-string-delete-newlines
               (buffer-string)))))
    ret))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-locale ()
  "Get locale information."
  (let* ((list
          '("LC_ALL"
            "LC_CTYPE"))
         val
         ret)
    (dolist (var list)
      (when (setq val (getenv var))
        (setq val (format "%s=%s" var val))
        (setq ret (if (null ret)
                      val
                    (concat ret ", " val)))))
    ret))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-os ()
  "Return OS information.
Debian Release: 3.0
Architecture: i386
Kernel: Linux terra 2.4.17 #1 Fri Feb 8 21:32:43 EET 2002 i686
Locale: LANG=en_US, LC_CTYPE=en_US."
  (let* ((kernel       (tinydebian-string-delete-newlines
                        (ti::process-uname)))
         (architecture (tinydebian-bug-system-info-os-architecture))
         (release      (tinydebian-bug-system-info-os-version))
         (locale       (tinydebian-bug-system-info-locale)))
    (format "\
Debian Release: %s
Architecture: %s
Kernel: %s
Locale: %s"
            release
            architecture
            kernel
            locale)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-severity ()
  "Select bug severity."
  (setq tinydebian-:severity-selected nil)
    (while (null tinydebian-:severity-selected)
      (ti::menu-menu 'tinydebian-:menu-severity)
      (unless tinydebian-:severity-selected
        (message "TinyDebian: Please select severity.")
        (sit-for 1)))
    tinydebian-:severity-selected)

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-report-mail-insert-details (info)
  "Insert Details for apckage INFO into Mail."
  (ti::mail-text-start 'move)
  (insert "Package: " (cdr (assoc "Package" info)) "\n")
  (insert "Version: " (cdr (assoc "Version" info)) "\n")
  (insert "Severity: " (tinydebian-bug-severity)   "\n\n")
  (let* ((point       (point))
         (depends     (tinydebian-bug-system-info-depends info "Depends"))
         (pre-depends (tinydebian-bug-system-info-depends info "Pre-Depends")))
    (insert "\n\n-- System Information\n"
            (tinydebian-bug-system-info-os)
            (format "\n\n-- Versions of packages `%s depends on'.\n"
                    (cdr (assoc "Package" info)))
            (if pre-depends
                (concat "Pre-Depends:\n" pre-depends)
              "")
            (if depends
                (concat "Depends:\n" depends)))
    (goto-char point)))


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tinydebian-bug-report-mail (info)
  "Submit Debian bug report. INFO is alist of attributes for package.
An example reportbug(1)' looks like

To: submit@bugs.debian.org
Subject: autolog ....
--text follows this line--
Package: autolog
Version: 0.35-10
Severity: wishlist



-- System Information
Debian Release: 3.0
Architecture: i386
Kernel: Linux foo 2.4.17 #1 Fri Feb 8 21:32:43 EET 2002 i686
Locale: LANG=en_US, LC_CTYPE=en_US

Versions of packages autolog depends on:
ii  cron                          3.0pl1-72  management of regular background p
ii  libc6                         2.2.5-3    GNU C Library: Shared libraries an


Subject: autolog based on DNS and IP names
Package: autolog
Version: 0.35-10
Severity: wishlist



-- System Information
Debian Release: 3.0
Architecture: i386
Kernel: Linux terra 2.4.17 #1 Fri Feb 8 21:32:43 EET 2002 i686
Locale: LANG=en_US, LC_CTYPE=en_US

Versions of packages autolog depends on:
ii  cron                          3.0pl1-72  management of regular background p
ii  libc6                         2.2.5-3    GNU C Library: Shared libraries an."
  (interactive
   (progn
     (if (y-or-n-p "[TinyDebian] Submit bug report? ")
         (list (tinydebian-package-info))
       nil)))
  (let ((status  (or (cdr-safe (assoc "Status" info)) ""))
        (package (or (cdr-safe (assoc "Package" info)) "")))
    (cond
     ((null info)
      (message "TinyDebian: no INFO available to send a bug report."))
     ((string-match "not-installed" status)
      (message "TinyDebian: bug report skipped. %s' status is [%s]"
               package status))
     (t
      (let* ((name   (format "*mail* Debian Bug %s" package))
             buffer)
        (cond
         ((and (setq buffer (get-buffer name))
               (null (y-or-n-p
                      "[TinyDebian] delete previous bug report? ")))
          (pop-to-buffer buffer))
         (t
          (pop-to-buffer (get-buffer-create name))
          (erase-buffer)
          (let ((subject (read-string "[TinyDebian] bug Subject: ")))
            (mail-setup
             "submit@bugs.debian.org" subject nil nil nil nil))
          (message-mode)
          (tinydebian-bug-report-mail-insert-details info))))))))

;;}}}
;;{{{ Admin functions: MAIL reports

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-command-audit-report-tiger-make-chmod (file line)
  "Make suotable chmod command for FILE according to LINE report."
  (let* ((operand "+")
         group
         group-cmd
         type
         type-cmd)
    (when (string-match
           "should .*+have +\\([^ \t\r\n]+\\) +\\([^ \t\r\n.]+\\)"
           line)
      (setq group (match-string 1 line)
            type  (match-string 2 line))
      (if (string-match "should not" line)
          (setq operand "-"))
      (cond
       ((string= group "group")
        (setq group-cmd "g"))
       ((string= group "world")
        (setq group-cmd "o")))
      (cond
       ((string-match type "read")
        (setq type-cmd "r"))
       ((string-match type "write")
        (setq type-cmd "w"))
       ((string-match type "exec")
        (setq type-cmd "x")))
      (when (and operand type-cmd group-cmd)
        (format "chmod %s%s%s %s;" group-cmd operand type-cmd file)))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-command-audit-report-tiger (beg end)
  "Process tiger(1) mail system report on region BEG END.
The body of mail looks like:

    # Performing check of system file permissions...
    OLD: --WARN-- [perm001w] /var/log/wtmp should not have group write.
    OLD: --WARN-- [perm001w] /var/run/utmp should not have group write.
    OLD: --WARN-- [perm001w] /var/log/XFree86.0.log should not have world read.

For which a corresponding command to correct the error is generated.

    chmod g-w /var/log/wtmp;
    chmod g-w /var/run/utmp;
    chmod o-r /var/log/XFree86.0.log;

You can select region and these commands to shell `sh' with command
`shell-command-on-region' which can be called with \\[shell-command-on-region]."
  (interactive "r")
  (let* ((buffer (get-buffer-create tinydebian-:buffer-tiger))
         done
         file
         str)
    (goto-char beg)
    (while (re-search-forward
            "--WARN-- +[^ \t\r\n]+ +\\(\\([^ \t\r\n]+\\).*\\)"
            nil end)
      (setq file (match-string 2)
            str  (match-string 1))
      (unless done                      ;Draw one empty line between calls
        (setq done t)
        (ti::append-to-buffer buffer "\n"))
      (when (setq str (tinydebian-command-audit-report-tiger-make-chmod
                       file str))
        (ti::append-to-buffer buffer (concat str "\n"))))
    (cond
     ((ti::buffer-empty-p buffer)
      (message
       "TinyDebian: Hm, region did not have --WARN-- chmod candidates."))
     (t
      (display-buffer buffer)
      (message
       (substitute-command-keys
        (concat
         "TinyDebian: [tiger] "
         "Select region and send commands to"
         " `sh' with \\[shell-command-on-region]")))))))

;;}}}

(tinydebian-install-severity-functions) ;; Auto-created functions

(add-hook 'tinydebian-:bts-mode-define-keys-hook
          'tinydebian-bts-mode-define-keys)

(provide   'tinydebian)
(run-hooks 'tinydebian-:load-hook)

;;; tinydebian.el ends here
